#ifndef Application_h
#define Application_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseApplication.h"
#include "CommandLineArguments.h"
#include "Config.h"
#include "Counter.h"
#include "IconEngine.h"

#include <QBasicTimer>

class SystemTrayIcon;
class StateFrame;
class LogDialog;
class MessageQueue;

//* Main Window singleton object
class Application: public BaseApplication, public Counter
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    Application( CommandLineArguments );

    //* destructor
    virtual ~Application( void );


    //*@name modifier
    //@{

    //* create all widgets
    bool realizeWidget( void );

    //@}

    //*@name accessors
    //@{

    //* command line parser
    CommandLineParser commandLineParser( CommandLineArguments = CommandLineArguments(), bool ignoreWarnings = true ) const;

    //* toggle log action
    QAction& showLogAction( void ) const
    { return *showLogAction_; }

    //* usage
    void usage( void ) const;

    //* application name
    virtual QString applicationName( void ) const
    { return "Qssh"; }

    //* application icon
    virtual QIcon applicationIcon( void ) const
    { return IconEngine::get( ":/qssh.png" ); }

    // application version
    virtual QString applicationVersion( void ) const
    { return VERSION; }

    //@}

    protected:

    //* timer event
    virtual void timerEvent( QTimerEvent* );

    protected Q_SLOTS:

    //* configuration
    void _configuration( void );

    //* update message queue geometry
    void _updateMessageQueue( void );

    //* process request from application manager
    virtual bool _processCommand( Server::ServerCommand );

    private:

    //* cvs log display frame
    SystemTrayIcon *systemTrayIcon_ = nullptr;

    //* message queue
    MessageQueue* messageQueue_ = nullptr;

    //* log dialog
    LogDialog* logDialog_ = nullptr;

    //* toggle log dialog action
    QAction* showLogAction_ = nullptr;

    //* connection timer
    QBasicTimer timer_;

};

#endif
