#ifndef ConnectionAttributesModel_h
#define ConnectionAttributesModel_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ListModel.h"
#include "Counter.h"
#include "SshConnectionAttributes.h"

#include <QIcon>

//* tunnel attributes model
class ConnectionAttributesModel: public ListModel<Ssh::ConnectionAttributes>, public Counter
{

    Q_OBJECT

    public:

    //* column types
    enum ColumnTypes
    {
        Name,
        Host,
        User,
        nColumns
    };

    //* constructor
    ConnectionAttributesModel( QObject* = nullptr );

    //* flags
    virtual Qt::ItemFlags flags( const QModelIndex& ) const;

    //* data
    virtual QVariant data(const QModelIndex&, int) const;

    //* header data
    virtual QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const;

    //* number of columns for a given index
    virtual int columnCount( const QModelIndex& = QModelIndex() ) const
    { return nColumns; }


    //* mime type
    virtual QStringList mimeTypes( void ) const;

    //* mime data
    virtual QMimeData* mimeData( const QModelIndexList& ) const;

    //* drop mine data
    virtual bool dropMimeData(const QMimeData*, Qt::DropAction, int, int, const QModelIndex&);

    //@}

    Q_SIGNALS:

    //* emitted via drag and drop
    void itemOrderChanged( int, int );

    protected:

    //* sort
    virtual void _sort( int, Qt::SortOrder = Qt::AscendingOrder )
    {}

    private:

    //* list column names
    static const QString columnTitles_[nColumns];

    //* icon
    QIcon icon_;

};

#endif
