#ifndef ConnectionListWidget_h
#define ConnectionListWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionAttributesModel.h"
#include "SshConnectionAttributes.h"

#include <QPushButton>

class SimpleListView;

//* handles ssh connection attributes
class ConnectionListWidget: public QWidget
{

    Q_OBJECT

    public:

    //* constructor
    ConnectionListWidget( QWidget* = nullptr );

    //*@name accessors
    //*{

    //* get connection attributes
    Ssh::ConnectionAttributes::List attributes( void ) const;

    //* add button
    QPushButton& addButton( void ) const
    { return *addButton_; }

    //* add button
    QPushButton& editButton( void ) const
    { return *editButton_; }

    //* model
    const ConnectionAttributesModel& model( void ) const
    { return model_; }

    //* list
    SimpleListView& list( void ) const
    { return *list_; }

    //* unicity
    bool isUnique( const Ssh::ConnectionAttributes& ) const;

    //*}

    //*@name modifiers
    //*{

    //* set connection attributes
    void setAttributes( const Ssh::ConnectionAttributes::List& );

    //* model
    ConnectionAttributesModel& model( void )
    { return model_; }

    //@}

    protected Q_SLOTS:

    //* update button enability
    void _updateButtons( void );

    //* remove
    void _remove( void );

    //* move up selected task
    void _up( void );

    //* move down selected task
    void _down( void );

    //* reorder
    void _reorder( int, int );

    private:

    //* unicity
    bool _checkUnicity( Ssh::ConnectionAttributes& );

    //* tunnel list
    SimpleListView* list_ = nullptr;

    //* tunnel attributes model
    ConnectionAttributesModel model_;

    //* add
    QPushButton* addButton_ = nullptr;

    //* edit
    QPushButton* editButton_ = nullptr;

    //* remove
    QPushButton* removeButton_ = nullptr;

    //* move up
    QPushButton* moveUpButton_ = nullptr;

    //* move down
    QPushButton* moveDownButton_ = nullptr;

};

#endif
