#ifndef MainMenu_h
#define MainMenu_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "SshConnectionAttributes.h"
#include "Counter.h"
#include "Debug.h"
#include "Severity.h"

#include <QBasicTimer>
#include <QMenu>
#include <QSet>
#include <QTimerEvent>

//* application main window
class MainMenu: public QMenu, public Counter
{

    Q_OBJECT

    public:

    //* constructor;
    MainMenu( QWidget* = nullptr );

    //@name accessors
    //@{

    //* return number of connections
    int connectionsCount( void ) const;

    //* configuration
    QAction& configurationAction( void ) const
    { return *configurationAction_; }

    //@}

    //@name modifiers
    //@{

    //* read connections
    virtual void readConnections( void );

    //* auto connect
    virtual void autoConnect( void );

    using QMenu::connect;
    using QMenu::disconnect;

    //* connect given connection
    virtual void connect( QString );

    //* disconnect given connection
    virtual void disconnect( QString );

    //* reconnect given connection
    virtual void reconnect( QString );

    //@}

    Q_SIGNALS:

    //* message available
    void messageAvailable( QString, Severity = Default );

    //* connection status changed
    void connectionStatusChanged( void );

    public Q_SLOTS:

    //* reconnect
    void reconnectExisting( void );

    //* reconnect all
    void reconnectAll( void );

    //* close all connections
    void closeConnections( void );

    protected:

    //* show event
    virtual void showEvent( QShowEvent* );

    //* timer event
    void timerEvent( QTimerEvent* );

    //* list of connected devices
    using DeviceSet = QSet<QString>;
    DeviceSet _connectedDevices( void ) const;

    //* check whether current device is active and select one if not
    void _checkDevice( void );

    private Q_SLOTS:

    //* update configuration
    virtual void _updateConfiguration( void );

    //* connection have been changed and needs to be saved to disk
    void _saveConnections( void );

    //* configure connections
    void _configureConnections( void );

    private:

    //* add connection
    void _addConnection( const Ssh::ConnectionAttributes& );

    //* actions
    void _installActions( void );

    //* device timer
    QBasicTimer deviceTimer_;

    //* route file name, used to get devices list
    QString routeFileName_;

    //* connected device
    QString device_;

    //* configuration action
    QAction* configurationAction_ = nullptr;

    //* reconnection
    QAction* reconnectAction_ = nullptr;

    //* same connection settings
    class SameConnectionFTor
    {

        public:

        //* constructor
        SameConnectionFTor( const Ssh::ConnectionAttributes& attributes ):
            attributes_( attributes )
        {}

        //* predicate
        bool operator() (const Ssh::ConnectionAttributes& ) const;

        private:

        //* prediction
        Ssh::ConnectionAttributes attributes_;

    };


};

#endif
