/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MessageQueue.h"

#include "Debug.h"
#include "MessageBox.h"
#include "Singleton.h"
#include "SvgEngine.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>


//______________________________________________________________
MessageQueue::MessageQueue( QObject* parent ):
    QObject( parent ),
    Counter( "MessageQueue" )
{
    Debug::Throw( "MessageQueue::MessageQueue.\n" );
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
}


//______________________________________________________________
void MessageQueue::setVerticalOffset( int value )
{ verticalOffset_ = value; }

//______________________________________________________________
void MessageQueue::processMessage( QString message )
{
    Debug::Throw() << "MessageQueue::processMessage - " << message << endl;
    if( !messageBox_ ) _createMessageBox();
    messageBox_->addMessage( message );
    messageBox_->hideFromTaskBar();
    _updateGeometry();
    messageBox_->show();
}

//______________________________________________________________
void MessageQueue::_createMessageBox( void )
{

    // create dialog
    messageBox_ = new MessageBox( 0, "Notification from ssh tunnel daemon" );
    connect( messageBox_, SIGNAL(destroyed()), SLOT(_messageBoxDestroyed()) );

}

//______________________________________________________________
void MessageQueue::_updateGeometry( void )
{
    QSize size(
        qMax( messageBox_->sizeHint().width(), messageBox_->minimumSize().width() ),
        qMax( messageBox_->sizeHint().height(), messageBox_->minimumSize().height() ) );

    // get desktop size
    const QSize desktopSize( QApplication::desktop()->screenGeometry( QApplication::desktop()->screenNumber( messageBox_ ) ).size() );

    int x( 0 );
    int y( 0 );
    if( direction_ == Descending )
    {
        y = verticalOffset_;
        x = desktopSize.width()-size.width();

    } else {

        y = verticalOffset_;
        y-= size.height();
        x = desktopSize.width()-size.width();

    }

    Debug::Throw() << "MessageQueue::processMessage - position: (" << x << "," << y << ")" << endl;
    messageBox_->setGeometry( QRect( QPoint( x,y ), size ) );

}

//____________________________________________________
void MessageQueue::_messageBoxDestroyed( void )
{
    Debug::Throw( "MessageQueue::_updateMessages.\n" );
    messageBox_ = 0;
}

//______________________________________________________________
void MessageQueue::_updateConfiguration( void )
{
    Debug::Throw( "MessageQueue::_updateConfiguration.\n" );

    // reload svg engine (needed for dialogs)
    Svg::SvgEngine::get().reload( true );

}
