#ifndef SystemTrayIcon_h
#define SystemTrayIcon_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "Severity.h"

#include <QSystemTrayIcon>

class MainMenu;
class Menu;
class ToolTipWidget;
class SystemNotifications;

//* application system tray icon
class SystemTrayIcon: public QSystemTrayIcon, public Counter
{

    Q_OBJECT;

    public:

    //* constructor
    SystemTrayIcon( QObject* = nullptr );

    //* destructor
    virtual ~SystemTrayIcon( void );

    //* main window
    MainMenu& mainMenu( void ) const
    { return *mainMenu_; }

    //* menu
    Menu& menu( void ) const
    { return *contextMenu_; }

    //* returns true if system notifications are supported
    bool supportsSystemNotifications( void ) const;

    Q_SIGNALS:

    //* message available
    void messageAvailable( QString );

    protected:

    //* generic event handler
    virtual bool event( QEvent* );

    public Q_SLOTS:

    //* notify
    void notify( QString, Severity = Default );

    protected Q_SLOTS:

    //* activated
    void _activate( QSystemTrayIcon::ActivationReason );

    //* update tooltip
    void _updateToolTip( void );

    private:

    //* show menu
    void _showMenu( QMenu* ) const;

    //* system notification
    SystemNotifications* systemNotifications_ = nullptr;

    //* tooltip
    ToolTipWidget* toolTipWidget_ = nullptr;

    //* main widget
    MainMenu* mainMenu_ = nullptr;

    //* menu
    Menu* contextMenu_ = nullptr;

};

#endif
